@@ -101,8 +101,9 @@ static Panel* panel_allocate(void)
     p->visible = TRUE;
     p->height_when_hidden = 2;
     p->transparent = 0;
-    p->alpha = 127;
-    p->tintcolor = 0xFFFFFFFF;
+    p->alpha = 255;
+    gdk_color_parse("white", &p->gtintcolor);
+    p->tintcolor = gcolor2rgb24(&p->gtintcolor);
     p->usefontcolor = 0;
     p->fontcolor = 0x00000000;
     p->spacing = 0;
@@ -819,29 +820,48 @@ static void panel_set_visibility(Panel *p, gboolean visible)
 
 static gboolean panel_leave_real(Panel *p)
 {
+    /* If the pointer is grabbed by this application, leave the panel displayed.
+     * There is no way to determine if it is grabbed by another application, such as an application that has a systray icon. */
     if (gdk_display_pointer_is_grabbed(p->display))
         return TRUE;
 
+    /* If the pointer is inside the panel, leave the panel displayed. */
     gint x, y;
     gdk_display_get_pointer(p->display, NULL, &x, &y, NULL);
     if ((p->cx <= x) && (x <= (p->cx + p->cw)) && (p->cy <= y) && (y <= (p->cy + p->ch)))
         return TRUE;
 
+    /* If the panel is configured to autohide and if it is visible, hide the panel. */
     if ((p->autohide) && (p->visible))
         panel_set_visibility(p, FALSE);
 
+    /* Clear the timer. */
     p->hide_timeout = 0;
     return FALSE;
 }
 
 static gboolean panel_enter(GtkImage *widget, GdkEventCrossing *event, Panel *p)
 {
-    if (p->hide_timeout)
-        return FALSE;
-
-    p->hide_timeout = g_timeout_add(500, (GSourceFunc) panel_leave_real, p);
-
-    panel_set_visibility(p, TRUE);
+    /* We may receive multiple enter-notify events when the pointer crosses into the panel.
+     * Do extra tests to make sure this does not cause misoperation such as blinking.
+     * If the pointer is inside the panel, unhide it. */
+    gint x, y;
+    gdk_display_get_pointer(p->display, NULL, &x, &y, NULL);
+    if ((p->cx <= x) && (x <= (p->cx + p->cw)) && (p->cy <= y) && (y <= (p->cy + p->ch)))
+    {
+        /* If the pointer is inside the panel and we have not already unhidden it, do so and
+         * set a timer to recheck it in a half second. */
+        if (p->hide_timeout == 0)
+        {
+            p->hide_timeout = g_timeout_add(500, (GSourceFunc) panel_leave_real, p);
+            panel_set_visibility(p, TRUE);
+        }
+    }
+    else
+    {
+        /* If the pointer is not inside the panel, simulate a timer expiration. */
+        panel_leave_real(p);
+    }
     return TRUE;
 }
 
@@ -881,6 +901,19 @@ void panel_image_set_from_file(Panel * p, GtkWidget * image, char * file)
     }
 }
 
+/* Set an image from a icon theme with scaling to the panel icon size. */
+gboolean panel_image_set_icon_theme(Panel * p, GtkWidget * image, const gchar * icon)
+{
+    if (gtk_icon_theme_has_icon(gtk_icon_theme_get_default(), icon))
+    {
+        GdkPixbuf * pixbuf = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), icon, p->icon_size, 0, NULL);
+        gtk_image_set_from_pixbuf(GTK_IMAGE(image), pixbuf);
+        g_object_unref(pixbuf);
+        return TRUE;
+    }
+    return FALSE;
+}
+
 static void
 panel_start_gui(Panel *p)
 {
